@@ -14,21 +14,24 @@ from bardolph.vm.instruction import Instruction, OpCode
 from bardolph.vm.vm_codes import JumpCondition, LoopVar, Operand, Operator
 from bardolph.vm.vm_codes import Register, SetOp
 
-assembly = [
+_assembly = [
     #instructions
 
 ]
 
-params_0 = (OpCode.BREAKPOINT, OpCode.COLOR, OpCode.END_LOOP, OpCode.LOOP,
-            OpCode.NOP, OpCode.PAUSE, OpCode.POWER, OpCode.STOP, OpCode.WAIT)
-
-params_1 = (OpCode.END, OpCode.GET_COLOR, OpCode.JSR, OpCode.OP, OpCode.POP,
-            OpCode.PUSH, OpCode.PUSHQ, OpCode.ROUTINE)
+_param_counts = {op_code: 0 for op_code in (OpCode.BREAKPOINT, OpCode.COLOR,
+    OpCode.DISC, OpCode.END_LOOP, OpCode.GET_COLOR, OpCode.LOOP, OpCode.NOP,
+    OpCode.PAUSE, OpCode.STOP, OpCode.POWER, OpCode.WAIT)}
+_param_counts.update({op_code: 1 for op_code in (OpCode.DISCM, OpCode.DNEXT,
+    OpCode.END, OpCode.JSR, OpCode.OP, OpCode.POP, OpCode.PUSH, OpCode.PUSHQ,
+    OpCode.ROUTINE)})
+_param_counts.update({op_code: 2 for op_code in (OpCode.CONSTANT, OpCode.DNEXTM,
+    OpCode.JUMP, OpCode.MOVE, OpCode.MOVEQ, OpCode.PARAM, OpCode.TIME_PATTERN)})
 
 def get_assembly():
     current_instruction = 0
-    while current_instruction < len(assembly):
-        value = assembly[current_instruction]
+    while current_instruction < len(_assembly):
+        value = _assembly[current_instruction]
         current_instruction += 1
         yield value
 
@@ -37,12 +40,7 @@ def build_instructions():
     it = iter(get_assembly())
     op_code = next(it, None)
     while op_code is not None:
-        if op_code in params_0:
-            param_count = 0
-        elif op_code in params_1:
-            param_count = 1
-        else:
-            param_count = 2
+        param_count = _param_counts[OpCode(op_code)]
         param0 = None if param_count < 1 else next(it)
         param1 = None if param_count < 2 else next(it)
         program.append(Instruction(op_code, param0, param1))
@@ -50,20 +48,20 @@ def build_instructions():
     return program
 
 def main():
+    injection.configure()
+
     ap = argparse.ArgumentParser()
     ap.add_argument(
-        '-d', '--debug', help='do debug-level logging', action='store_true')
+        '-v', '--verbose', help='do debug-level logging', action='store_true')
     ap.add_argument(
         '-f', '--fakes', help='use fake lights', action='store_true')
     arg_helper.add_n_argument(ap)
     args = ap.parse_args()
 
-    injection.configure()
-    settings_init = settings.use_base(config_values.functional)
     overrides = {
         'sleep_time': 0.1
     }
-    if args.debug:
+    if args.verbose:
         overrides['log_level'] = logging.DEBUG
         overrides['log_to_console'] = True
     if args.fakes:
@@ -72,6 +70,7 @@ def main():
     if n_arg is not None and not args.fakes:
         overrides.update(n_arg)
 
+    settings_init = settings.using(config_values.functional)
     settings_init.add_overrides(overrides).configure()
     light_module.configure()
     machine.Machine().run(build_instructions())
