@@ -27,8 +27,6 @@ def init_args():
     parser.add_argument(
         '-f', '--fakes', help='use fake lights', action='store_true')
     arg_helper.add_n_argument(parser)
-    parser.add_argument(
-        '-r', '--repeat', help='repeat until key pressed', action='store_true')
     parser.add_argument(
         '-s', '--script', help='run script from command line', action='store')
     parser.add_argument(
@@ -47,7 +45,7 @@ def init_settings(args):
     if args.fakes:
         overrides['use_fakes'] = True
 
-    settings_init = settings.use_base(
+    settings_init = settings.using(
         config_values.functional).add_overrides(overrides)
 
     if args.config_file:
@@ -56,12 +54,7 @@ def init_settings(args):
     overrides = arg_helper.get_overrides(args)
     if overrides is not None:
         settings_init.add_overrides(overrides)
-
-    ini = os.getenv('BARDOLPH_INI')
-    if ini:
-        settings_init.apply_file(ini)
-
-    settings_init.configure()
+    settings_init.apply_env().configure()
 
 
 def main():
@@ -72,9 +65,9 @@ def main():
 
     jobs = job_control.JobControl()
     if args.script is not None:
-        jobs.add_job(ScriptJob.from_string(args.script), args.repeat)
+        jobs.add_job(ScriptJob.from_string(args.script))
     for file_name in args.file:
-        jobs.add_job(ScriptJob.from_file(file_name), args.repeat)
+        jobs.add_job(ScriptJob.from_file(file_name))
 
 
 if __name__ == "__main__":
