@@ -1,4 +1,5 @@
 import configparser
+import os
 
 from . import i_lib
 from . import injection
@@ -9,7 +10,7 @@ class Settings:
 
     def __init__(self):
         self._config = Settings._the_config
-        
+
     def __contains__(self, name):
         return name in self._config
 
@@ -28,23 +29,29 @@ class Builder:
     def apply_file(self, file_name):
         if self._config is None:
             self._config = {}
-        config = configparser.ConfigParser()
-        config.read(file_name)
-        for section in config.sections():
-            for key in config[section]:
-                value = config[section][key]
+        file_config = configparser.ConfigParser()
+        file_config.read(file_name)
+        for section in file_config.sections():
+            for key in file_config[section]:
+                value = file_config[section][key]
                 self._config[key] = {
                     'True': True,
                     'False': False
                 }.get(value, value)
         return self
 
+    def apply_env(self, env_name='BARDOLPH_INI'):
+        ini = os.getenv(env_name)
+        if ini:
+            self.apply_file(ini)
+        return self
+
     def configure(self):
         Settings._the_config = self._config
         injection.bind(Settings).to(i_lib.Settings)
-        
 
-def use_base(initial=None):
+
+def using(initial=None):
     return Builder(initial)
 
 
