@@ -24,8 +24,8 @@ class Parser:
         self._lexer = None
         self._error_output = ''
         self._call_context = CallContext()
-        self._current_token_type = None
-        self._current_token = None
+        self._current_token_type = TokenTypes.NO_TOKEN
+        self._current_token = ''
         self._op_code = OpCode.NOP
         self._code_gen = CodeGen()
         self._command_map = {
@@ -35,38 +35,38 @@ class Parser:
             TokenTypes.GET: self._get,
             TokenTypes.IF: self._if,
             TokenTypes.NAME: self._call_routine,
+            TokenTypes.NO_TOKEN: self._syntax_error,
             TokenTypes.OFF: self._power_off,
             TokenTypes.ON: self._power_on,
             TokenTypes.PAUSE: self._pause,
+            TokenTypes.PRINT: self._print,
+            TokenTypes.PRINTLN: self._print,
             TokenTypes.REGISTER: self._set_reg,
             TokenTypes.REPEAT: self._repeat,
             TokenTypes.SET: self._set,
             TokenTypes.UNITS: self._set_units,
-            TokenTypes.WAIT: self._wait
+            TokenTypes.WAIT: self._wait,
+            None: self._syntax_error
         }
         self._token_trace = False
 
-    def parse(self, input_string, optimize=False):
+    def parse(self, input_string):
         self._call_context.clear()
         self._code_gen.clear()
         self._error_output = ''
         self._lexer = lex.Lex(input_string)
-        self._next_token()
+        self.next_token()
         success = self._script()
         self._lexer = None
-        if success:
-            if optimize:
-                self._code_gen.optimize()
-            return self._code_gen.program
-        return None
+        return self._code_gen.program if success else None
 
-    def load(self, file_name, optimize=False):
+    def load(self, file_name):
         logging.debug('File name: {}'.format(file_name))
         try:
             srce = open(file_name, 'r')
             input_string = srce.read()
             srce.close()
-            return self.parse(input_string, optimize)
+            return self.parse(input_string)
         except FileNotFoundError:
             logging.error('Error: file {} not found.'.format(file_name))
         except OSError:
@@ -76,15 +76,12 @@ class Parser:
         return self._error_output
 
     @property
-    def current_token(self) -> str:
-        return self._current_token
+    def current_token(self):
+        return self._current_token or ''
 
     @property
-    def current_token_type(self) -> TokenTypes:
-        return self._current_token_type
-
-    def next_token(self) -> bool:
-        return self._next_token()
+    def current_token_type(self):
+        return self._current_token_type or TokenTypes.NO_TOKEN
 
     def _script(self) -> bool:
         return self._body() and self._eof()
@@ -97,10 +94,10 @@ class Parser:
 
     def _eof(self) -> bool:
         if self._current_token_type != TokenTypes.EOF:
-            return self._trigger_error("Didn't get to end of file.")
+            return self.trigger_error("Didn't get to end of file.")
         return True
 
-    def _command(self) -> bool:
+    def _command(self):
         return self._command_map.get(
             self._current_token_type, self._syntax_error)()
 
@@ -111,16 +108,16 @@ class Parser:
         if reg == Register.TIME:
             return self._time()
 
-        self._next_token()
+        self.next_token()
         if self._current_token_type == TokenTypes.LITERAL_STRING:
             return self._string_to_reg(reg)
         return self.rvalue(reg)
 
     def _string_to_reg(self, reg) -> bool:
         if reg != Register.NAME:
-            return self._trigger_error('Quoted value not allowed here.')
+            return self.trigger_error('Quoted value not allowed here.')
         self._add_instruction(OpCode.MOVEQ, self._current_token, reg)
-        return self._next_token()
+        return self.next_token()
 
     def _set(self):
         return self._action(OpCode.COLOR)
@@ -137,7 +134,7 @@ class Parser:
         """ op_code: COLOR or POWER """
         self._op_code = op_code
         self._add_instruction(OpCode.WAIT)
-        self._next_token()
+        self.next_token()
         if self._current_token_type == TokenTypes.ALL:
             return self._all_operand()
         return self._operand_list()
@@ -146,17 +143,19 @@ class Parser:
         self._add_instruction(OpCode.MOVEQ, None, Register.NAME)
         self._add_instruction(OpCode.MOVEQ, Operand.ALL, Register.OPERAND)
         self._add_instruction(self._op_code)
-        return self._next_token()
+        return self.next_token()
 
     def _operand_list(self) -> bool:
-        """ For every operand in the list, issue the instruction in
-        self._op_code. """
+        """
+        For every operand in the list, issue the instruction in
+        self._op_code.
+        """
         if not self._operand():
             return False
         self._add_instruction(self._op_code)
 
         while self._current_token_type == TokenTypes.AND:
-            self._next_token()
+            self.next_token()
             if not self._operand():
                 return False
             self._add_instruction(self._op_code)
@@ -169,17 +168,17 @@ class Parser:
         """
         if self._current_token_type == TokenTypes.GROUP:
             operand = Operand.GROUP
-            self._next_token()
+            self.next_token()
         elif self._current_token_type == TokenTypes.LOCATION:
             operand = Operand.LOCATION
-            self._next_token()
+            self.next_token()
         else:
             operand = Operand.LIGHT
 
         const_str = self._current_str()
         if const_str is not None:
             self._add_instruction(OpCode.MOVEQ, const_str, Register.NAME)
-            self._next_token()
+            self.next_token()
         elif self._current_token_type == TokenTypes.NAME:
             if not self._var_operand():
                 return False
@@ -196,16 +195,16 @@ class Parser:
 
     def _var_operand(self) -> bool:
         if not self._call_context.has_symbol_typed(
-                self._current_token, SymbolType.VAR):
-            return self.token_error('Not a variable: "{}"')
+                self._current_token, SymbolType.MACRO, SymbolType.VAR):
+            return self.token_error('Neither variable nor macro: "{}"')
         self._add_instruction(OpCode.MOVE, self._current_token, Register.NAME)
-        return self._next_token()
+        return self.next_token()
 
     def _zone_range(self) -> bool:
         if self._op_code != OpCode.COLOR:
-            return self._trigger_error('Zones not supported for {}'.format(
-                self._op_code.name.tolower()))
-        self._next_token()
+            return self.trigger_error('Zones not supported for {}'.format(
+                self._op_code.name.lower()))
+        self.next_token()
         return self._set_zones()
 
     def _set_zones(self, only_one=False):
@@ -225,32 +224,32 @@ class Parser:
         return True
 
     def _set_units(self) -> bool:
-        self._next_token()
+        self.next_token()
         mode = {
             TokenTypes.RAW: UnitMode.RAW,
             TokenTypes.LOGICAL: UnitMode.LOGICAL
         }.get(self._current_token_type, None)
 
         if mode is None:
-            return self._trigger_error(
+            return self.trigger_error(
                 'Invalid parameter "{}" for units.'.format(self._current_token))
 
         self._add_instruction(OpCode.MOVEQ, mode, Register.UNIT_MODE)
-        return self._next_token()
+        return self.next_token()
 
     def _wait(self) -> bool:
         self._add_instruction(OpCode.WAIT)
-        return self._next_token()
+        return self.next_token()
 
     def _get(self) -> bool:
-        self._next_token()
+        self.next_token()
         if not self.at_rvalue():
             return self.token_error('Needed light for get, got "{}".')
         if not self.rvalue(Register.NAME):
             return False
 
         if self._current_token_type == TokenTypes.ZONE:
-            self._next_token()
+            self.next_token()
             if not self._set_zones(True):
                 return False
             operand = Operand.MZ_LIGHT
@@ -263,11 +262,22 @@ class Parser:
 
     def _pause(self):
         self._add_instruction(OpCode.PAUSE)
-        self._next_token()
+        self.next_token()
+        return True
+
+    def _print(self):
+        add_newline = self._current_token_type == TokenTypes.PRINTLN
+        self.next_token()
+        while self._current_token_type.is_printable():
+            if not self.rvalue():
+                return self.token_error('Expected something to print, got {}')
+            self._code_gen.add_instruction(OpCode.OUT, Register.RESULT)
+        if add_newline:
+            self._code_gen.add_instruction(OpCode.OUTQ, Operand.NULL)
         return True
 
     def _time(self):
-        self._next_token()
+        self.next_token()
         if self._current_token_type == TokenTypes.AT:
             self.next_token()
             return self._process_time_patterns()
@@ -279,35 +289,37 @@ class Parser:
             return self._time_spec_error()
         self._add_instruction(
             OpCode.TIME_PATTERN, SetOp.INIT, time_pattern)
-        self._next_token()
+        self.next_token()
 
         while self._current_token_type == TokenTypes.OR:
-            self._next_token()
+            self.next_token()
             time_pattern = self._current_time_pattern()
             if time_pattern is None:
                 return self._time_spec_error()
             self._add_instruction(
                 OpCode.TIME_PATTERN, SetOp.UNION, time_pattern)
-            self._next_token()
+            self.next_token()
 
         return True
 
     def _assignment(self) -> bool:
-        self._next_token()
+        self.next_token()
         if self._current_token_type != TokenTypes.NAME:
             return self.token_error('Expected name for assignment, got "{}"')
         dest_name = self._current_token
-        self._next_token()
+        self.next_token()
         if not self.rvalue(dest_name):
             return False
         self._call_context.add_variable(dest_name)
         return True
 
-    def rvalue(self, dest=Register.RESULT) -> bool:
+    def rvalue(self, dest=Register.RESULT, code_gen=None) -> bool:
         """
         Consume the current token as an rvalue, generating the code to evaluate
         it and to move the result into dest.
         """
+        if code_gen is None:
+            code_gen = self._code_gen
         move_inst = OpCode.MOVE
         value = self._current_constant()
         if value is not None:
@@ -322,15 +334,15 @@ class Parser:
             parser = ExprParser(self._current_token)
             if not parser.generate_code(self._code_gen):
                 return self.token_error('Error parsing expression "{}"')
-            self._add_instruction(OpCode.POP, Register.RESULT)
+            code_gen.add_instruction(OpCode.POP, Register.RESULT)
             value = Register.RESULT
         elif self._current_token_type == TokenTypes.REGISTER:
             value = self._current_reg()
         else:
             return self.token_error('Cannot use {} as a value.')
 
-        self._add_instruction(move_inst, value, dest)
-        return self._next_token()
+        code_gen.add_instruction(move_inst, value, dest)
+        return self.next_token()
 
     def at_rvalue(self) -> bool:
         if self._current_token_type not in (
@@ -354,14 +366,14 @@ class Parser:
         return not self._call_context.has_routine(self.current_token)
 
     def _definition(self) -> bool:
-        self._next_token()
+        self.next_token()
         if self._current_token_type != TokenTypes.NAME:
             return self.token_error('Expected name for definition, got: {}')
         name = self._current_token
 
-        self._next_token()
+        self.next_token()
         if self._detect_routine_start():
-            if self._call_context.get_routine(name) is not None:
+            if not self._call_context.get_routine(name).undefined:
                 return self.token_error('Already defined: "{}"')
             return self._routine_definition(name)
 
@@ -373,7 +385,7 @@ class Parser:
         to a command, or the name of an existing routine, it's defining a new
         routine and not a variable.
         """
-        if self._call_context.get_routine(self._current_token) is not None:
+        if self._call_context.has_routine(self._current_token):
             return True
         return self._current_token_type in (
             TokenTypes.BEGIN, TokenTypes.WITH,
@@ -388,16 +400,17 @@ class Parser:
         """
         value = self._current_literal()
         if value is None:
-            value = self._call_context.get_macro(self._current_token)
-        if value is None:
-            return self.token_error('Macro needs constant, got "{}"')
+            inner_macro = self._call_context.get_macro(self._current_token)
+            if inner_macro is None:
+                return self.token_error('Macro needs constant, got "{}"')
+            value = inner_macro.value
         self._call_context.add_global(name, SymbolType.MACRO, value)
         self._add_instruction(OpCode.CONSTANT, name, value)
-        return self._next_token()
+        return self.next_token()
 
     def _routine_definition(self, name):
         if self._call_context.in_routine():
-            return self._trigger_error('Nested definition not allowed.')
+            return self.trigger_error('Nested definition not allowed.')
 
         self._call_context.enter_routine()
         self._call_context.push()
@@ -406,7 +419,7 @@ class Parser:
         routine = Routine(name)
         self._call_context.add_routine(routine)
         if self._current_token_type == TokenTypes.WITH:
-            self._next_token()
+            self.next_token()
             if not self._param_decl(routine):
                 return False
         result = self.command_seq()
@@ -424,7 +437,7 @@ class Parser:
         name = self._current_token
         routine.add_param(name)
         self._call_context.add_variable(name)
-        self._next_token()
+        self.next_token()
         while (self._current_token_type == TokenTypes.NAME and not
                 self._call_context.has_routine(self._current_token)):
             name = self._current_token
@@ -433,7 +446,7 @@ class Parser:
                 return None
             routine.add_param(name)
             self._call_context.add_variable(name)
-            self._next_token()
+            self.next_token()
         return True
 
     def command_seq(self):
@@ -442,21 +455,21 @@ class Parser:
         return self.compound_command()
 
     def compound_command(self):
-        self._next_token()
+        self.next_token()
         while self._current_token_type != TokenTypes.END:
             if self._current_token_type == TokenTypes.EOF:
-                return self._trigger_error('End of file before "end".')
+                return self.trigger_error('End of file before "end".')
             if not self._command():
                 return False
-        return self._next_token()
+        return self.next_token()
 
     def _call_routine(self):
         routine = self._call_context.get_routine(self._current_token)
-        if routine is None:
+        if routine.undefined:
             return self.token_error('Unknown name: "{}"')
 
-        self._next_token()
-        for param_name in routine.params:
+        self.next_token()
+        for param_name in routine.value.params:
             if not self.rvalue():
                 return False
             self._add_instruction(OpCode.PARAM, param_name, Register.RESULT)
@@ -465,15 +478,15 @@ class Parser:
         return True
 
     def _if(self) -> bool:
-        self._next_token()
+        self.next_token()
         if self._current_token_type != TokenTypes.EXPRESSION:
             return self.token_error('Unable to use as condition: "{}"')
         parser = ExprParser(self._current_token)
         if not parser.generate_code(self._code_gen):
             return self.token_error('Error parsing expression "{}"')
         self._add_instruction(OpCode.POP, Register.RESULT)
-        marker = self._code_gen.if_start()
-        self._next_token()
+        marker = self._code_gen.if_true_start()
+        self.next_token()
         if not self.command_seq():
             return False
         if self.current_token_type == TokenTypes.ELSE:
@@ -493,7 +506,7 @@ class Parser:
     def _add_message(self, message):
         self._error_output += '{}\n'.format(message)
 
-    def _trigger_error(self, message):
+    def trigger_error(self, message):
         full_message = 'Line {}: {}'.format(
             self._lexer.get_line_number(), message)
         self._add_message(full_message)
@@ -529,7 +542,8 @@ class Parser:
             return value
         if self._current_token_type != TokenTypes.NAME:
             return None
-        return self._call_context.get_macro(self._current_token)
+        macro = self._call_context.get_macro(self._current_token)
+        return None if macro.undefined else macro.value
 
     def _current_int(self):
         value = self._current_constant()
@@ -553,26 +567,21 @@ class Parser:
 
     def _current_time_pattern(self) -> TimePattern:
         """
-        Returns the current token as a time pattern. If the current token
-        isn't a time pattern or Symbol, returns None.
-
-        Only literals or macros. Can't yet pass a time pattern as a
-        parameter to a routine.
+        Returns the current token as a time pattern. Only literals or macros.
         """
         if self._current_token_type == TokenTypes.TIME_PATTERN:
             return TimePattern.from_string(self._current_token)
         if self._current_token_type == TokenTypes.NAME:
-            return self._call_context.get_macro(self._current_token)
-        return None
+            return self._call_context.get_macro(self._current_token).value
+        return TimePattern(None, None)
 
     def _current_reg(self):
         if self._current_token_type != TokenTypes.REGISTER:
             return None
         return Register.from_string(self._current_token)
 
-    def _next_token(self):
-        (self._current_token_type,
-         self._current_token) = self._lexer.next_token()
+    def next_token(self):
+        self._current_token_type, self._current_token = self._lexer.next_token()
         if self._token_trace:
             logging.info(
                 'Next token: "{}" ({})'.format(
@@ -583,7 +592,7 @@ class Parser:
         self._code_gen.add_instruction(OpCode.BREAKPOINT)
 
     def token_error(self, message_format):
-        return self._trigger_error(message_format.format(self._current_token))
+        return self.trigger_error(message_format.format(self._current_token))
 
     def _unimplementd(self):
         return self.token_error('Unimplemented at token "{}"')
@@ -598,15 +607,13 @@ class Parser:
 def main():
     arg_parser = argparse.ArgumentParser()
     arg_parser.add_argument('file', help='name of the script file')
-    arg_parser.add_argument(
-        '-o', '--optimize', help='enable optimization', action='store_true')
     args = arg_parser.parse_args()
 
     logging.basicConfig(
         level=logging.DEBUG,
         format='%(filename)s(%(lineno)d) %(funcName)s(): %(message)s')
     parser = Parser()
-    output_code = parser.load(args.file, args.optimize)
+    output_code = parser.load(args.file)
     if output_code:
         inst_num = 0
         for inst in output_code:
