@@ -5,6 +5,7 @@ from ..lib.auto_repl import auto
 class TokenTypes(Enum):
     ALL = auto()
     AND = auto()
+    AS = auto()
     ASSIGN = auto()
     AT = auto()
     BEGIN = auto()
@@ -19,17 +20,21 @@ class TokenTypes(Enum):
     GET = auto()
     GROUP = auto()
     IF = auto()
+    IN = auto()
+    LIGHTS = auto()
     LITERAL_STRING = auto()
     LOCATION = auto()
     LOGICAL = auto()
     NAME = auto()
+    NO_TOKEN = auto()
     NUMBER = auto()
     OFF = auto()
     ON = auto()
     OR = auto()
     POWER = auto()
+    PRINT = auto()
+    PRINTLN = auto()
     PAUSE = auto()
-    RANGE = auto()
     RAW = auto()
     REGISTER = auto()
     REPEAT = auto()
@@ -38,7 +43,6 @@ class TokenTypes(Enum):
     TIME_PATTERN = auto()
     TO = auto()
     UNITS = auto()
-    UNTIL = auto()
     UNKNOWN = auto()
     WHILE = auto()
     WITH = auto()
@@ -57,3 +61,13 @@ class TokenTypes(Enum):
                 TokenTypes.SET,
                 TokenTypes.UNITS,
                 TokenTypes.WAIT)
+
+    def is_command(self):
+        return self in TokenTypes.commands()
+
+    def is_printable(self):
+        return self in (
+            TokenTypes.EXPRESSION, TokenTypes.LITERAL_STRING, TokenTypes.NAME,
+            TokenTypes.NUMBER, TokenTypes.REGISTER, TokenTypes.TIME_PATTERN,
+            TokenTypes.UNKNOWN
+        )
