@@ -11,28 +11,33 @@ from bardolph.controller.units import UnitMode
 
 from .call_stack import CallStack
 from .loader import Loader
-from .vm_codes import JumpCondition, OpCode, Operand, Register, SetOp
+from .vm_codes import JumpCondition, LoopVar, OpCode, Operand, Register, SetOp
+from .vm_discover import VmDiscover
 from .vm_math import VmMath
 
 class Registers:
     def __init__(self):
-        self.hue = 0
-        self.saturation = 0
-        self.brightness = 0
-        self.kelvin = 0
-        self.duration = 0
-        self.first_zone = None
-        self.last_zone = None
+        self.hue = 0.0
+        self.saturation = 0.0
+        self.brightness = 0.0
+        self.kelvin = 0.0
+        self.disc_forward = False
+        self.duration = 0.0
+        self.first_zone = 0
+        self.last_zone = 0
         self.power = False
         self.result = None
         self.name = None
         self.operand = None
-        self.time = 0  # ms.
+        self.time = 0.0  # ms.
         self.unit_mode = UnitMode.LOGICAL
 
-    def get_color(self) -> [int]:
+    def get_color(self):
         return [self.hue, self.saturation, self.brightness, self.kelvin]
 
+    def store_color(self, color):
+        self.hue, self.saturation, self.brightness, self.kelvin = color
+
     def get_by_enum(self, reg):
         return getattr(self, reg.name.lower())
 
@@ -47,6 +52,8 @@ class Registers:
 
 
 class Machine:
+    _MAX_PRINTBUF = 2048
+
     def __init__(self):
         self._pc = 0
         self._cue_time = 0
@@ -55,12 +62,19 @@ class Machine:
         self._program = []
         self._reg = Registers()
         self._call_stack = CallStack()
+        self._print_buffer = ''
         self._vm_math = VmMath(self._call_stack, self._reg)
+        self._vm_discover = VmDiscover(self._call_stack, self._reg)
         self._enable_pause = True
         self._keep_running = True
         self._fn_table = {}
-        for opcode in (OpCode.COLOR,
+        for opcode in (OpCode.BREAKPOINT,
+                       OpCode.COLOR,
                        OpCode.CONSTANT,
+                       OpCode.DISC,
+                       OpCode.DISCM,
+                       OpCode.DNEXT,
+                       OpCode.DNEXTM,
                        OpCode.END,
                        OpCode.END_LOOP,
                        OpCode.GET_COLOR,
@@ -71,6 +85,8 @@ class Machine:
                        OpCode.MOVEQ,
                        OpCode.NOP,
                        OpCode.OP,
+                       OpCode.OUT,
+                       OpCode.OUTQ,
                        OpCode.PARAM,
                        OpCode.PAUSE,
                        OpCode.PUSH,
@@ -108,22 +124,8 @@ class Machine:
             if inst.op_code not in (OpCode.END, OpCode.JSR, OpCode.JUMP):
                 self._pc += 1
         self._clock.stop()
-
-    def interpret(self, input_stream) -> None:
-        fn_table = self._fn_table.copy()
-        for op_code in (OpCode.END,
-                        OpCode.END_LOOP,
-                        OpCode.JSR,
-                        OpCode.JUMP,
-                        OpCode.LOOP,
-                        OpCode.PARAM):
-            fn_table[op_code] = self._nop
-        self._clock.start()
-        for inst in input_stream:
-            if inst.op_code == OpCode.STOP:
-                break
-            self._fn_table[inst.op_code]()
-        self._clock.stop()
+        if len(self._print_buffer) > 0:
+            print(self._print_buffer.rstrip())
 
     def stop(self) -> None:
         self._keep_running = False
@@ -133,7 +135,7 @@ class Machine:
         reg = self._reg
         reg.hue, reg.saturation, reg.brightness, reg.kelvin = color
 
-    def color_from_reg(self) -> [int]:
+    def color_from_reg(self):
         return self._reg.get_color()
 
     def get_variable(self, name):
@@ -154,7 +156,7 @@ class Machine:
     @inject(LightSet)
     def _color_all(self, light_set=injected) -> None:
         color = self._assure_raw_color(self._reg.get_color())
-        duration = self._assure_raw(Register.DURATION, self._reg.duration)
+        duration = self._assure_raw_time(self._reg.duration)
         light_set.set_color(color, duration)
 
     @inject(LightSet)
@@ -165,7 +167,7 @@ class Machine:
         else:
             light.set_color(
                 self._assure_raw_color(self._reg.get_color()),
-                self._assure_raw(Register.DURATION, self._reg.duration))
+                self._assure_raw_time(self._reg.duration))
 
     @inject(LightSet)
     def _color_mz_light(self, light_set=injected) -> None:
@@ -180,40 +182,42 @@ class Machine:
             light.set_zone_color(
                 start_index, end_index + 1,
                 self._assure_raw_color(self._reg.get_color()),
-                self._assure_raw(Register.DURATION, self._reg.duration))
+                self._assure_raw_time(self._reg.duration))
 
     @inject(LightSet)
     def _color_group(self, light_set=injected) -> None:
-        lights = light_set.get_group(self._reg.name)
-        if lights is None:
+        light_names = light_set.get_group(self._reg.name)
+        if light_names is None:
             logging.warning("Unknown group: {}".format(self._reg.name))
         else:
-            self._color_multiple(lights)
+            self._color_multiple(
+                [light_set.get_light(name) for name in light_names])
 
     @inject(LightSet)
     def _color_location(self, light_set=injected) -> None:
-        lights = light_set.get_location(self._reg.name)
-        if lights is None:
+        light_names = light_set.get_location(self._reg.name)
+        if light_names is None:
             logging.warning("Unknown location: {}".format(self._reg.name))
         else:
-            self._color_multiple(lights)
+            self._color_multiple(
+                [light_set.get_light(name) for name in light_names])
 
     def _color_multiple(self, lights) -> None:
         color = self._assure_raw_color(self._reg.get_color())
-        duration = self._assure_raw(Register.DURATION, self._reg.duration)
+        duration = self._assure_raw_time(self._reg.duration)
         for light in lights:
             light.set_color(color, duration)
 
     def _power(self) -> None: {
-        Operand.ALL: self._power_all,
-        Operand.LIGHT: self._power_light,
-        Operand.GROUP: self._power_group,
-        Operand.LOCATION: self._power_location
-    }[self._reg.operand]()
+            Operand.ALL: self._power_all,
+            Operand.LIGHT: self._power_light,
+            Operand.GROUP: self._power_group,
+            Operand.LOCATION: self._power_location
+        }[self._reg.operand]()
 
     @inject(LightSet)
     def _power_all(self, light_set=injected) -> None:
-        duration = self._assure_raw(Register.DURATION, self._reg.duration)
+        duration = self._assure_raw_time(self._reg.duration)
         light_set.set_power(self._reg.get_power(), duration)
 
     @inject(LightSet)
@@ -222,26 +226,28 @@ class Machine:
         if light is None:
             Machine._report_missing(self._reg.name)
         else:
-            duration = self._assure_raw(Register.DURATION, self._reg.duration)
+            duration = self._assure_raw_time(self._reg.duration)
             light.set_power(self._reg.get_power(), duration)
 
     @inject(LightSet)
     def _power_group(self, light_set=injected) -> None:
-        lights = light_set.get_group(self._reg.name)
-        if lights is None:
+        light_names = light_set.get_group(self._reg.name)
+        if light_names is None:
             logging.warning(
                 'Power invoked for unknown group "{}"'.format(self._reg.name))
         else:
-            self._power_multiple(light_set.get_group(self._reg.name))
+            self._power_multiple(
+                [light_set.get_light(name) for name in light_names])
 
     @inject(LightSet)
     def _power_location(self, light_set=injected) -> None:
-        lights = light_set.get_location(self._reg.name)
-        if lights is None:
+        light_names = light_set.get_location(self._reg.name)
+        if light_names is None:
             logging.warning(
                 "Power invoked for unknown location: {}".format(self._reg.name))
         else:
-            self._power_multiple(lights)
+            self._power_multiple(
+                [light_set.get_light(name) for name in light_names])
 
     def _power_multiple(self, lights) -> None:
         power = self._reg.get_power()
@@ -258,9 +264,9 @@ class Machine:
                 if self._zone_check(light):
                     zone = self._reg.first_zone
                     color = light.get_color_zones(zone, zone + 1)[0]
-                    self.color_to_reg(self._maybe_logical_color(color))
+                    self.color_to_reg(self._maybe_converted_color(color))
             else:
-                self.color_to_reg(self._maybe_logical_color(light.get_color()))
+                self.color_to_reg(self._maybe_converted_color(light.get_color()))
 
     def _param(self) -> None:
         """
@@ -311,27 +317,64 @@ class Machine:
 
     def _nop(self) -> None: pass
 
-    def _push(self):
-        return self._vm_math.push(self.current_inst.param0)
+    def _push(self) -> None:
+        self._vm_math.push(self.current_inst.param0)
 
-    def _pushq(self):
-        return self._vm_math.pushq(self.current_inst.param0)
+    def _pushq(self) -> None:
+        self._vm_math.pushq(self.current_inst.param0)
 
-    def _pop(self):
-        return self._vm_math.pop(self.current_inst.param0)
+    def _pop(self) -> None:
+        self._vm_math.pop(self.current_inst.param0)
 
-    def _op(self):
-        return self._vm_math.op(self.current_inst.param0)
+    def _op(self) -> None:
+        self._vm_math.op(self.current_inst.param0)
 
-    def _bin_op(self, operator):
-        return self._vm_math.bin_op(operator)
+    def _bin_op(self, operator) -> None:
+        self._vm_math.bin_op(operator)
 
-    def _unary_op(self, operator):
-        return self._vm_math.unary_op(operator)
+    def _unary_op(self, operator) -> None:
+        self._vm_math.unary_op(operator)
+
+    def _disc(self) -> None:
+        self._vm_discover.disc()
+
+    def _discm(self) -> None:
+            self._vm_discover.discm(self.current_inst.param0)
+
+    def _dnext(self) -> None:
+        self._vm_discover.dnext(self.current_inst.param0)
+
+    def _dnextm(self) -> None:
+        self._vm_discover.dnextm(
+            self.current_inst.param0, self.current_inst.param1)
+
+    def _out(self) -> None:
+        value = self.current_inst.param0
+        if isinstance(value, Register):
+            value = self._reg.get_by_enum(value)
+        elif isinstance(value, (str, LoopVar)):
+            value = self._call_stack.get_variable(value)
+        self._print_buffer += str(value) + ' '
+        self._check_pbuf()
+
+    def _outq(self) -> None:
+        value = self.current_inst.param0
+        if value == Operand.NULL:
+            print(self._print_buffer.rstrip())
+            self._print_buffer = ''
+        else:
+            self._print_buffer += str(value) + ' '
+            self._check_pbuf()
+
+    def _check_pbuf(self) -> None:
+        if len(self._print_buffer) > self._MAX_PRINTBUF:
+            print(self._print_buffer, end='')
+            self._print_buffer = ''
 
     def _pause(self) -> None:
         if self._enable_pause:
-            print("Press any to continue, q to quit, ! to run.")
+            print("Press any key to continue, q to quit, "
+                  + "! to run without stopping again.")
             char = getch()
             if char == 'q':
                 self.stop()
@@ -340,7 +383,7 @@ class Machine:
                 if char == '!':
                     self._enable_pause = False
 
-    def _constant(self):
+    def _constant(self) -> None:
         name = self.current_inst.param0
         value = self.current_inst.param1
         self._call_stack.put_constant(name, value)
@@ -354,92 +397,77 @@ class Machine:
                 time /= 1000.0
             self._clock.pause_for(time)
 
-    def _assure_raw(self, reg, value) -> int:
-        """
-        If in logical mode, convert incoming value to raw units. If not in
-        logical mode, no conversion is necessary.
-        """
-        if (self._reg.unit_mode == UnitMode.LOGICAL
-                and units.requires_conversion(reg)):
-            return units.as_raw(reg, value)
-        if (self._reg.unit_mode == UnitMode.RAW and reg == Register.HUE
-                and value > 65535):
-            value %= 65536
+    def _assure_raw_time(self, value) -> int:
+        if self._reg.unit_mode == UnitMode.LOGICAL:
+            return units.time_raw(value)
         return value
 
-    def _assure_raw_color(self, color) -> [int]:
+    def _assure_raw_color(self, color):
         if self._reg.unit_mode == UnitMode.RAW:
             return color
-        result = 4 * [0]
-        result[0] = units.as_raw(Register.HUE, color[0])
-        result[1] = units.as_raw(Register.SATURATION, color[1])
-        result[2] = units.as_raw(Register.BRIGHTNESS, color[2])
-        result[3] = round(color[3])
-        return result
+        return units.logical_to_raw(color)
 
-    def _maybe_logical(self, reg, value) -> int:
+    def _maybe_converted_color(self, color):
         """
-        If in logical mode, convert incoming value to logical units. If not
-        in logical mode, or for some registers, no conversion is necessary.
-
-        Typically, the incoming value comes from a register, which always
-        contains a raw value.
+        The incoming color always consists of raw values.
         """
-        if (self._reg.unit_mode == UnitMode.LOGICAL
-                and units.requires_conversion(reg)):
-            return units.as_logical(reg, value)
-        if reg == Register.HUE and value > 360:
-            value %= 360
-        return value
-
-    def _maybe_logical_color(self, color) -> [int]:
         if self._reg.unit_mode == UnitMode.RAW:
             return color
-        result = 4 * [0]
-        result[0] = units.as_logical(Register.HUE, color[0])
-        result[1] = units.as_logical(Register.SATURATION, color[1])
-        result[2] = units.as_logical(Register.BRIGHTNESS, color[2])
-        result[3] = color[3]
-        return result
+        elif self._reg.unit_mode == UnitMode.LOGICAL:
+            return units.raw_to_logical(color)
+        return units.raw_to_rgb(color)
 
-    def _move(self) -> bool:
+    def _move(self) -> None:
         """
         Move from variable/register to variable/register.
         """
         inst = self.current_inst
-        srce = inst.param0
+        value = inst.param0
         dest = inst.param1
-        if isinstance(srce, Register):
-            value = self._reg.get_by_enum(srce)
-        else:
-            value = self._call_stack.get_variable(srce)
-            if value is None:
-                return self._trigger_error('Unknown: "{}"'.format(srce))
-        return self._do_put_value(dest, value)
+        if isinstance(value, Register):
+            value = self._reg.get_by_enum(value)
+        elif isinstance(value, (str, LoopVar)):
+            value = self._call_stack.get_variable(value)
+        self._do_put_value(dest, value)
 
-    def _moveq(self) -> bool:
+    def _moveq(self) -> None:
         """
         Move a value from the instruction itself into a register or variable.
         """
         value = self.current_inst.param0
         dest = self.current_inst.param1
         if dest == Register.UNIT_MODE:
-            if self._reg.unit_mode != value:
-                fn = (units.as_logical if value == UnitMode.LOGICAL
-                      else units.as_raw)
-                self._reg.hue = fn(Register.HUE, self._reg.hue)
-                self._reg.saturation = fn(
-                    Register.SATURATION, self._reg.saturation)
-                self._reg.brightness = fn(
-                    Register.BRIGHTNESS, self._reg.brightness)
-        return self._do_put_value(dest, value)
+            self._switch_unit_mode(value)
+        self._do_put_value(dest, value)
 
-    def _do_put_value(self, dest, value) -> bool:
+    def _switch_unit_mode(self, to_mode) -> None:
+        from_mode = self._reg.unit_mode
+        if from_mode == to_mode:
+            return
+
+        key = lambda mode0, mode1: str(mode0) + str(mode1)
+        converters = (
+            (UnitMode.LOGICAL, UnitMode.RAW, units.logical_to_raw),
+            (UnitMode.LOGICAL, UnitMode.RGB, units.logical_to_rgb),
+            (UnitMode.RGB, UnitMode.RAW, units.rgb_to_raw),
+            (UnitMode.RGB, UnitMode.LOGICAL, units.rgb_to_logical),
+            (UnitMode.RAW, UnitMode.RGB, units.raw_to_rgb),
+            (UnitMode.RAW, UnitMode.LOGICAL, units.raw_to_logical))
+        convert = {key(from_mode, to_mode): fn
+              for from_mode, to_mode, fn in converters}[key(from_mode, to_mode)]
+        self._reg.store_color(convert(self._reg.get_color()))
+        if to_mode == UnitMode.RAW:
+            self._reg.duration, self._reg.time = (units.time_raw(t)
+                for t in (self._reg.duration, self._reg.time))
+        elif from_mode == UnitMode.RAW:
+            self._reg.duration , self._reg.time = (units.time_logical(t)
+                for t in (self._reg.duration, self._reg.time))
+
+    def _do_put_value(self, dest, value) -> None:
         if isinstance(dest, Register):
             self._reg.set_by_enum(dest, value)
         else:
             self._call_stack.put_variable(dest, value)
-        return True
 
     def _time_pattern(self) -> None:
         inst = self.current_inst
@@ -456,15 +484,14 @@ class Machine:
         return True
 
     @classmethod
-    def _report_missing(cls, name):
+    def _report_missing(cls, name) -> None:
         logging.warning("Light \"{}\" not found.".format(name))
 
     def _power_param(self):
         return 65535 if self._reg.power else 0
 
-    @classmethod
-    def _breakpoint(cls):
-        breakpoint()
+    def _breakpoint(self) -> None:
+        print("At breakpoint.")
 
     def _trigger_error(self, message) -> bool:
         logging.error(message)
